SystemOrganization addCategory: #'Stack-Exercise'!

Object subclass: #OOStack
	instanceVariableNames: 'innerList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!OOStack class methodsFor: 'error descriptions' stamp: 'HernanWilkinson 5/7/2012 11:51'!
stackEmptyErrorDescription
	
	^ 'Stack is empty'! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'JuanManuelTastzian 5/6/2013 21:06'!
initialize
	innerList := LinkedList new! !

!OOStack methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/6/2013 21:10'!
isEmpty
	^innerList isEmpty! !

!OOStack methodsFor: 'associating' stamp: 'JuanManuelTastzian 5/6/2013 21:12'!
pop
	(innerList isEmpty) ifTrue: [ ^Error signal: 'Stack is empty' ].
	^innerList removeFirst! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'JuanManuelTastzian 5/6/2013 21:06'!
push: anObject
	innerList addFirst: anObject! !

!OOStack methodsFor: 'as yet unclassified' stamp: 'JuanManuelTastzian 5/6/2013 21:13'!
size
	^innerList size! !

!OOStack methodsFor: 'accessing' stamp: 'JuanManuelTastzian 5/6/2013 21:13'!
top
	(innerList isEmpty) ifTrue: [ ^Error signal: 'Stack is empty' ].
	^innerList first! !

Object subclass: #StackElement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

StackElement subclass: #Base
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!Base methodsFor: 'as yet unclassified' stamp: 'JuanManuelTastzian 5/6/2013 21:46'!
initialize
	value := nil! !

!Base methodsFor: 'as yet unclassified' stamp: 'JuanManuelTastzian 5/6/2013 21:47'!
pop
	^Error signal: 'Stack is empty'! !

!Base methodsFor: 'as yet unclassified' stamp: 'JuanManuelTastzian 5/6/2013 21:47'!
size
	^0! !

!Base methodsFor: 'as yet unclassified' stamp: 'JuanManuelTastzian 5/6/2013 21:47'!
top
	^Error signal: 'Stack is empty'! !

StackElement subclass: #Element
	instanceVariableNames: 'nextValue value previousElement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!Element methodsFor: 'as yet unclassified' stamp: 'JuanManuelTastzian 5/6/2013 21:51'!
pop 
	^value! !

!Element methodsFor: 'as yet unclassified' stamp: 'JuanManuelTastzian 5/6/2013 21:54'!
size 
	^1 + previousElement size! !

!Element methodsFor: 'as yet unclassified' stamp: 'JuanManuelTastzian 5/6/2013 21:50'!
top 
	^value! !

!StackElement methodsFor: 'as yet unclassified' stamp: 'JuanManuelTastzian 5/6/2013 21:48'!
pop
	^self subclassResponsibility! !

!StackElement methodsFor: 'as yet unclassified' stamp: 'JuanManuelTastzian 5/6/2013 21:49'!
size
	^self subclassResponsibility! !

!StackElement methodsFor: 'as yet unclassified' stamp: 'JuanManuelTastzian 5/6/2013 21:48'!
top
	^self subclassResponsibility! !

TestCase subclass: #OOStackTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:30'!
test01StackShouldBeEmptyWhenCreated

	| stack |
	
	stack := OOStack new.
	
	self assert: stack isEmpty! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:30'!
test02PushAddElementsToTheStack

	| stack |
	
	stack := OOStack new.
	stack push: 'Something'.
	
	self deny: stack isEmpty! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:31'!
test03PopRemovesElementsFromTheStack

	| stack |
	
	stack := OOStack new.
	stack push: 'Something'.
	stack pop.
	
	self assert: stack isEmpty! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/8/2012 08:20'!
test04PopReturnsLastPushedObject

	| stack pushedObject |
	
	stack := OOStack new.
	pushedObject := 'Something'.
	stack push: pushedObject.
	
	self assert: stack pop = pushedObject! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:33'!
test05StackBehavesLIFO

	| stack firstPushedObject secondPushedObject |
	
	stack := OOStack new.
	firstPushedObject := 'First'.
	secondPushedObject := 'Second'.
	
	stack push: firstPushedObject.
	stack push: secondPushedObject.
	
	self assert: stack pop = secondPushedObject.
	self assert: stack pop = firstPushedObject.
	self assert: stack isEmpty 
	! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:35'!
test06TopReturnsLastPushedObject

	| stack pushedObject |
	
	stack := OOStack new.
	pushedObject := 'Something'.
	
	stack push: pushedObject.
	
	self assert: stack top = pushedObject.
	! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:36'!
test07TopDoesNotRemoveObjectFromStack

	| stack pushedObject |
	
	stack := OOStack new.
	pushedObject := 'Something'.
	
	stack push: pushedObject.
	
	self assert: stack size = 1.
	stack top.
	self assert: stack size = 1.
	! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:38'!
test08CanNotPopWhenThereAreNoObjectsInTheStack

	| stack  |
	
	stack := OOStack new.
	self
		should: [ stack pop ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText = OOStack stackEmptyErrorDescription ]
		
! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/16/2012 16:53'!
test09CanNotPopWhenThereAreNoObjectsInTheStackAndTheStackHadObjects

	| stack  |
	
	stack := OOStack new.
	stack push: 'Something'.
	stack pop.
	
	self
		should: [ stack pop ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText = OOStack stackEmptyErrorDescription ]
		
! !

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/16/2012 16:53'!
test10CanNotTopWhenThereAreNoObjectsInTheStack

	| stack  |
	
	stack := OOStack new.
	self
		should: [ stack top ]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: anError messageText = OOStack stackEmptyErrorDescription ]
		
! !
